@@ -16,44 +16,70 @@ from .settings import ALLOWED_PARAMS, CONTROL_PARAMS,\
                       TARGET_LANG_DE, TARGET_LANG_EN
 
 
+PROTOCOLS = re.compile(r'https?://')
+SELECTION = re.compile(r'(https?://[^">&? ]+?)(?:https?://)|(?:https?://[^/]+?/[^/]+?[&?]u(rl)?=)(https?://[^"> ]+)')
+MIDDLE_URL = re.compile(r'https?://.+?(https?://.+?)(?:https?://|$)')
+
+NETLOC_RE = re.compile(r'(?<=\w):(?:80|443)')
+PATH1 = re.compile(r'/+')
+PATH2 = re.compile(r'^(?:/\.\.(?![^/]))+')
+
+
 def clean_url(url, language=None):
     '''Helper function: chained scrubbing and normalization'''
     try:
         return normalize_url(scrub_url(url), language)
-    except ValueError:
+    except (AttributeError, ValueError):
         return None
 
 
 def scrub_url(url):
     '''Strip unnecessary parts and make sure only one URL is considered'''
     # trim
-    url = url.strip()
+    # https://github.com/cocrawler/cocrawler/blob/main/cocrawler/urls.py
+    # remove leading and trailing white space and unescaped control chars
+    url = url.strip('\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f'
+                    '\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f \r\n')
     # clean the input string
     url = url.replace('[ \t]+', '')
-    # trailing slashes
-    url = url.rstrip('/')
     # <![CDATA[http://www.urbanlife.de/item/260-bmw-i8-hybrid-revolution-unter-den-sportwagen.html]]>
     if url.startswith('<![CDATA['): # re.match(r'<!\[CDATA\[', url):
         url = url.replace('<![CDATA[', '') # url = re.sub(r'^<!\[CDATA\[', '', url)
         url = url.replace(']]>', '') # url = re.sub(r'\]\]>$', '', url)
+    # markup rests
+    url = re.sub(r'</?a>', '', url)
     # &amp;
     if '&amp;' in url:
         url = url.replace('&amp;', '&')
     #if '"' in link:
     #    link = link.split('"')[0]
     # double/faulty URLs
-    protocols = re.findall(r'https?://', url)
+    protocols = PROTOCOLS.findall(url)
     if len(protocols) > 1 and not 'web.archive.org' in url:
         logging.debug('double url: %s %s', len(protocols), url)
-        match = re.match(r'(https?://[^"> ]+?)(?:https?://)', url)
+        match = SELECTION.match(url)
         if match and validate_url(match.group(1))[0] is True:
-            logging.debug('taking url: %s', url)
             url = match.group(1)
+            logging.debug('taking url: %s', url)
         else:
-            match = re.match(r'https?://.+?(https?://.+?)(?:https?://|$)', url)
+            match = MIDDLE_URL.match(url)
             if match and validate_url(match.group(1))[0] is True:
                 url = match.group(1)
                 logging.debug('taking url: %s', url)
+    # too long and garbled URLs e.g. due to quotes URLs
+    # https://github.com/cocrawler/cocrawler/blob/main/cocrawler/urls.py
+    if len(url) > 500:  # arbitrary choice
+        match = re.match(r'(.*?)[<>"\'\r\n ]', url)
+        if match:
+            url = match.group(1)
+        if len(url) > 500:
+            logging.debug('invalid-looking link %s of length %d',
+                           url[:50] + '...', len(url))
+    # trailing ampersand
+    url = url.strip('&')
+    # trailing slashes in URLs without path or in embedded URLs
+    if url.count('/') == 3 or url.count('://') > 1:
+        url = url.rstrip('/')
     # lower
     # url = url.lower()
     return url
@@ -91,11 +117,11 @@ def normalize_url(parsed_url, strict=False, language=None):
         parsed_url = urlparse(parsed_url)
     # port
     if parsed_url.port is not None and parsed_url.port in (80, 443):
-        parsed_url = parsed_url._replace(netloc=re.sub(r'(?<=\w):(?:80|443)', '', parsed_url.netloc))
+        parsed_url = parsed_url._replace(netloc=NETLOC_RE.sub('', parsed_url.netloc))
     # path: https://github.com/saintamh/alcazar/blob/master/alcazar/utils/urls.py
-    newpath = re.sub(r'/+', '/', parsed_url.path)
+    newpath = PATH1.sub('/', parsed_url.path)
     # Leading /../'s in the path are removed
-    newpath = re.sub(r'^(?:/\.\.(?![^/]))+', '', newpath)
+    newpath = PATH2.sub('', newpath)
     # fragment
     if strict is True:
         newfragment = ''
@@ -107,7 +133,7 @@ def normalize_url(parsed_url, strict=False, language=None):
                  netloc=parsed_url.netloc.lower(),
                  path=newpath,
                  fragment=newfragment
-                 )    
+                 )
     # strip unwanted query elements
     parsed_url = clean_query(parsed_url, strict, language)
     # rebuild
